"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.enforceValidDependencies = enforceValidDependencies;
exports.dependencyPropTypes = dependencyPropTypes;
exports.calculate = calculate;
exports.calculateInitial = calculateInitial;
exports.calculateForDispatch = calculateForDispatch;
exports.makeDependencyIndex = makeDependencyIndex;

var _InspectStore = require("../store/InspectStore");

var _invariant = _interopRequireDefault(require("invariant"));

var _ObjectUtils = require("../utils/ObjectUtils");

var _Store = _interopRequireDefault(require("../store/Store"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function enforceValidDependencies(dependencies) {
  (0, _invariant.default)(dependencies && _typeof(dependencies) === 'object', 'expected `dependencies` to be an `object` but got `%s`', dependencies);
  (0, _ObjectUtils.oForEach)(dependencies, function (dependency, field) {
    if (dependency instanceof _Store.default) {
      return;
    }

    (0, _invariant.default)(dependency && _typeof(dependency) === 'object', 'expected `%s` to be an `object` but got `%s`', field, dependency);
    var deref = dependency.deref,
        stores = dependency.stores;
    (0, _invariant.default)(typeof deref === 'function', 'expected `%s.deref` to be a function but got `%s`', field, deref);
    (0, _invariant.default)(Array.isArray(stores), 'expected `%s.stores` to be an Array but got `%s`', field, dependency.stores);
    stores.forEach(function (store, index) {
      (0, _invariant.default)(store instanceof _Store.default, 'expected `%s.stores.%s` to be a `Store` but got `%s`', field, index, store);
    });
  });
  return dependencies;
}

function dependencyPropTypes(dependencies) {
  var existingPropTypes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var unrelatedPropTypes = (0, _ObjectUtils.oReduce)(existingPropTypes, function (keep, type, name) {
    if (!dependencies.hasOwnProperty(name)) {
      keep[name] = type;
    }

    return keep;
  }, {});
  return (0, _ObjectUtils.oReduce)(dependencies, function (types, dependency) {
    if (dependency instanceof _Store.default) {
      return types;
    } // eslint-disable-next-line react-app/react/forbid-foreign-prop-types


    var propTypes = dependency.propTypes;

    if (!propTypes || _typeof(propTypes) !== 'object') {
      return types;
    }

    return (0, _ObjectUtils.oMerge)(types, propTypes);
  }, unrelatedPropTypes);
}

function calculate(dependency, props, state) {
  if (dependency instanceof _Store.default) {
    return dependency.get();
  }

  var deref = dependency.deref,
      stores = dependency.stores;
  return deref(props, state, stores);
}

function calculateInitial(dependencies, props, state) {
  return (0, _ObjectUtils.oMap)(dependencies, function (dependency) {
    return calculate(dependency, props, state);
  });
}

function calculateForDispatch(dependencies, dependencyIndexEntry, props, state) {
  return (0, _ObjectUtils.oMap)(dependencyIndexEntry.fields, function (_, field) {
    return calculate(dependencies[field], props, state);
  });
}

function makeIndexEntry() {
  return {
    fields: {},
    dispatchTokens: {}
  };
}

function makeDependencyIndex(dependencies) {
  enforceValidDependencies(dependencies);
  return (0, _ObjectUtils.oReduce)(dependencies, function (index, dep, field) {
    var stores = dep instanceof _Store.default ? [dep] : dep.stores;
    stores.forEach(function (store) {
      (0, _InspectStore.getActionTypes)(store).forEach(function (actionType) {
        var entry = index[actionType];

        if (!entry) {
          entry = index[actionType] = makeIndexEntry();
        }

        var token = (0, _InspectStore.getDispatchToken)(store);
        entry.dispatchTokens[token] = true;
        entry.fields[field] = true;
      });
    });
    return index;
  }, {});
}