"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var DispatcherInstance = _interopRequireWildcard(require("../dispatcher/DispatcherInstance"));

var _DispatcherInterface = require("../dispatcher/DispatcherInterface");

var _invariant = _interopRequireDefault(require("invariant"));

var _Store = _interopRequireDefault(require("./Store"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HINT_LINK = 'Learn more about defining stores:' + ' https://github.com/HubSpot/general-store#create-a-store';

function enforceResponse(existingResponses, actionType, response) {
  (0, _invariant.default)(typeof actionType === 'string', 'StoreFactory.defineResponses: expected actionType to be a string' + ' but got "%s" instead. %s', actionType, HINT_LINK);
  (0, _invariant.default)(!existingResponses.hasOwnProperty(actionType), 'StoreFactory.defineResponses: conflicting resposes for actionType' + ' "%s". Only one response can be defined per actionType per Store. %s', actionType, HINT_LINK);
  (0, _invariant.default)(typeof response === 'function', 'StoreFactory.defineResponses: expected response to be a function' + ' but got "%s" instead. %s', response);
}

function defaultGetInitialState() {
  return undefined;
}

function defaultGetter(state) {
  return state;
}

var StoreFactory = /*#__PURE__*/function () {
  function StoreFactory() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        getter = _ref.getter,
        getInitialState = _ref.getInitialState,
        name = _ref.name,
        responses = _ref.responses;

    _classCallCheck(this, StoreFactory);

    _defineProperty(this, "_definition", void 0);

    this._definition = {
      getter: getter || defaultGetter,
      getInitialState: getInitialState || defaultGetInitialState,
      name: name,
      responses: responses || {}
    };
  }

  _createClass(StoreFactory, [{
    key: "defineGet",
    value: function defineGet(getter) {
      (0, _invariant.default)(this._definition.getter === defaultGetter, 'StoreFactory.defineGet: a getter is already defined.');
      return new StoreFactory(_objectSpread(_objectSpread({}, this._definition), {}, {
        getter: getter
      }));
    }
  }, {
    key: "defineGetInitialState",
    value: function defineGetInitialState(getInitialState) {
      (0, _invariant.default)(typeof getInitialState === 'function', 'StoreFactory.defineGetInitialState: getInitialState must be a function.');
      return new StoreFactory(_objectSpread(_objectSpread({}, this._definition), {}, {
        getInitialState: getInitialState
      }));
    }
  }, {
    key: "defineName",
    value: function defineName(name) {
      var currentName = this._definition.name;
      return new StoreFactory(_objectSpread(_objectSpread({}, this._definition), {}, {
        name: currentName ? "".concat(name, "(").concat(currentName, ")") : name
      }));
    }
  }, {
    key: "defineResponses",
    value: function defineResponses(newResponses) {
      var responses = this._definition.responses;
      (0, _invariant.default)(newResponses && _typeof(newResponses) === 'object', 'StoreFactory.defineResponses: newResponses must be an object');
      Object.keys(newResponses).forEach(function (actionType) {
        return enforceResponse(responses, actionType, newResponses[actionType]);
      });
      return new StoreFactory(_objectSpread(_objectSpread({}, this._definition), {}, {
        responses: _objectSpread(_objectSpread({}, responses), newResponses)
      }));
    }
  }, {
    key: "defineResponseTo",
    value: function defineResponseTo(actionTypes, response) {
      return this.defineResponses([].concat(actionTypes).reduce(function (responses, actionType) {
        responses[actionType] = response;
        return responses;
      }, {}));
    }
  }, {
    key: "getDefinition",
    value: function getDefinition() {
      return this._definition;
    }
  }, {
    key: "register",
    value: function register(dispatcher) {
      dispatcher = dispatcher || DispatcherInstance.get();
      (0, _invariant.default)(dispatcher !== null && _typeof(dispatcher) === 'object', "StoreFactory.register: you haven't provided a dispatcher instance." + ' You can pass an instance to' + ' GeneralStore.define().register(dispatcher) or use' + ' GeneralStore.DispatcherInstance.set(dispatcher) to set a global' + ' instance.' + ' https://github.com/HubSpot/general-store#default-dispatcher-instance');
      (0, _DispatcherInterface.enforceDispatcher)(dispatcher);
      var _this$_definition = this._definition,
          getter = _this$_definition.getter,
          getInitialState = _this$_definition.getInitialState,
          name = _this$_definition.name,
          responses = _this$_definition.responses;
      return new _Store.default({
        dispatcher: dispatcher,
        factory: this,
        getter: getter,
        initialState: getInitialState(),
        name: name,
        responses: responses
      });
    }
  }]);

  return StoreFactory;
}();

exports.default = StoreFactory;